<?php

defined('_JEXEC') or die('Restricted access');

use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Language\Text;

class plgJshoppingJshopping_privacy extends JPlugin {

    protected $autoloadLanguage = true;

    function __construct($subject, $config) {
        $componentName = JFactory::getApplication()->input->get('option', 'com_jshopping');
        if( $componentName != "com_jshopping" ){
            return 1;
        }
        JFactory::getLanguage()->load('plg_privacy_consents');
        $language = JFactory::getLanguage();
        $language->load('plg_privacy_consents', JPATH_ADMINISTRATOR, $language->getTag(), true);
        parent::__construct($subject, $config);
    }

    function onBeforeDisplayRegisterView(&$view) {
        $privacyArticleId = self::getPrivacyArticleId();
        $privacyArticleText = self::getPluginPrivacyParams()->privacy_note;
        if (!$privacyArticleText) {
            $privacyArticleText = 'PLG_SYSTEM_PRIVACYCONSENT_NOTE_FIELD_DEFAULT';
        }
        if ((int) $privacyArticleId > 0) {
            JHTML::_('behavior.modal', 'a.modal');
            $articlePrivacy = self::getArticleData($privacyArticleId);
            $link = SEFLink($articlePrivacy->link . '&tmpl=component');
            $href = "a href=" . $link;
        } else {
            $href = false;
        }

        $view->_tmpl_register_html_5 = '<div class="jshop_register">
            <h3>' . JText::_('PLG_SYSTEM_PRIVACYCONSENT_LABEL') . '</h3>
            <div class="alert alert-info">' . JText::_($privacyArticleText) . '</div>
            <div class="control-group">
                <div class="control-label">
                    <label id="jform_privacyconsent_privacy-lbl" for="jform_privacyconsent_privacy" class="hasPopover required" data-content="' . JText::_('PLG_SYSTEM_PRIVACYCONSENT_FIELD_DESC') . '" data-original-title="Privacy Policy"><a ' . $href . ' class="modal" >' . JText::_('PLG_SYSTEM_PRIVACYCONSENT_FIELD_LABEL') . '</a><span class="star">&nbsp;*</span></label>					
                </div>
                <div class="controls">
                    <fieldset id="jform_privacyconsent_privacy" class="required radio" required="required" aria-required="true">
                        <label for="jform_privacyconsent_privacy0" style="display: block;"> 
                            <input type="radio" id="jform_privacyconsent_privacy0" name="jform[privacyconsent][privacy]" value="1" required="" aria-required="true">
                            ' . JText::_('PLG_SYSTEM_PRIVACYCONSENT_OPTION_AGREE') . ' 
                        </label>
                        <label for="jform_privacyconsent_privacy1" style="display: block;">
                            <input type="radio" id="jform_privacyconsent_privacy1" name="jform[privacyconsent][privacy]" value="0" checked="checked" required="" aria-required="true">
                            ' . JText::_('JNO') . '
                        </label>
                    </fieldset>
                </div>
            </div>
        </div> ';
    }

    private function getArticleData($articleId) {
        (object) $article = new \stdClass();
        $db = JFactory::getDbo();
        $query = $db->getQuery(true)
                ->select($db->quoteName(array('id', 'alias', 'catid', 'language')))
                ->from($db->quoteName('#__content'))
                ->where($db->quoteName('id') . ' = ' . (int) $articleId);
        $db->setQuery($query);
        $article = $db->loadObject();

        JLoader::register('ContentHelperRoute', JPATH_BASE . '/components/com_content/helpers/route.php');

        $slug = $article->alias ? ($article->id . ':' . $article->alias) : $article->id;
        $article->link = ContentHelperRoute::getArticleRoute($slug, $article->catid, $article->language);
        return $article;
    }

    /**
     * Get privacy article ID. If the site is a multilingual website and there is associated article for the
     * current language, ID of the associlated article will be returned
     *
     * @return  integer
     *
     * @since   3.9.0
     */
    private function getPrivacyArticleId() {
        $privacyArticleId = self::getPluginPrivacyParams()->privacy_article;
        if ($privacyArticleId > 0 && JLanguageAssociations::isEnabled()) {
            $privacyAssociated = JLanguageAssociations::getAssociations('com_content', '#__content', 'com_content.item', $privacyArticleId);
            $currentLang = JFactory::getLanguage()->getTag();

            if (isset($privacyAssociated[$currentLang])) {
                $privacyArticleId = $privacyAssociated[$currentLang]->id;
            }
        }

        return $privacyArticleId;
    }

    private function getPluginPrivacyParams() {
        $plugin = JPluginHelper::getPlugin('system', 'privacyconsent');
        (object) $params = new \stdClass();
        $params = new JRegistry($plugin->params);
        $params->privacy_article = $params->get('privacy_article');
        $params->privacy_note = $params->get('privacy_note');
        return $params;
    }

    function onBeforeRegister(&$post, &$default_usergroup, &$userparams) {
        $mainframe = JFactory::getApplication();
        $return = SEFLink("index.php?option=com_jshopping&controller=user&task=register");
        $message = JText::_('PLG_SYSTEM_PRIVACYCONSENT_FIELD_ERROR');
        if ((int) $post['jform']['privacyconsent']['privacy'] == 0) {
            $mainframe->redirect($return, JText::_($message), "Error");
        }
    }

    function onAfterRegister(&$user, &$usershop, &$post, &$useractivation) {
        $this->app = JFactory::getApplication();
        $this->db = JFactory::getDbo();
        $data = $post;
        $userId = $user->id;

        // If user already consented before, no need to check it further
        if ($userId > 0 && self::isUserConsented($userId)) {
            return true;
        }

//        $option = $this->app->input->getCmd('option');
//        $task = $this->app->input->get->getCmd('task');
//        $form = $this->app->input->post->get('jform', array(), 'array');
        // Get the user's IP address
        $ip = $this->app->input->server->get('REMOTE_ADDR', '', 'string');

        // Get the user agent string
        $userAgent = $this->app->input->server->get('HTTP_USER_AGENT', '', 'string');

        // Create the user note
        $userNote = (object) array(
                    'user_id' => $userId,
                    'subject' => 'PLG_SYSTEM_PRIVACYCONSENT_SUBJECT',
                    'body' => Text::sprintf('PLG_SYSTEM_PRIVACYCONSENT_BODY', $ip, $userAgent),
                    'created' => JFactory::getDate()->toSql(),
        );

        try {
            $this->db->insertObject('#__privacy_consents', $userNote);
        } catch (Exception $e) {
            // Do nothing if the save fails
        }

        $message = array(
            'action' => 'consent',
            'id' => $userId,
            'title' => $user->username,
            'itemlink' => 'index.php?option=com_users&task=user.edit&id=' . $userId,
            'userid' => $userId,
            'username' => $user->username,
            'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $userId,
        );

        JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_actionlogs/models', 'ActionlogsModel');

        /* @var ActionlogsModelActionlog $model */
        $model = JModelLegacy::getInstance('Actionlog', 'ActionlogsModel');
        $model->addLog(array($message), 'PLG_SYSTEM_PRIVACYCONSENT_CONSENT', 'plg_system_privacyconsent', $userId);

        return true;
    }

    /**
     * Method to check if the given user has consented yet
     *
     * @param   integer  $userId  ID of uer to check
     *
     * @return  boolean
     *
     * @since   3.9.0
     */
    private function isUserConsented($userId) {
        $this->db = JFactory::getDbo();
        $query = $this->db->getQuery(true);
        $query->select('COUNT(*)')
                ->from('#__privacy_consents')
                ->where('user_id = ' . (int) $userId)
                ->where('subject = ' . $this->db->quote('PLG_SYSTEM_PRIVACYCONSENT_SUBJECT'))
                ->where('state = 1');
        $this->db->setQuery($query);

        return (int) $this->db->loadResult() > 0;
    }

}
